/**
\file AutoAnchorComms.h
AutoAnchor communications and control library

The auto anchor communications library is implemented using a background thread
to handle the sending and receiving of commands. This means that commands will
not block the user interface, and once communications has started, the main application
need not worry about the communications.

If the status of the auto anchor changes, this is optionally indicated asyncronously
by the StatusChangeFunc function. The application may also choose to regularly poll
the status of the auto anchor

*/

/**
\mainpage Overview

This library provides an interface layer between an auto-anchor device connected
through either a serial port or a USB port and your application.

\section including_sec Including in your application
To use the library, include AutoAnchorComms.h and UsbDirect.h in your application,
and then link with aainterface.lib

\section distrib_sec Distributing with your application
Aainterface.dll must be distributed with your application. If you wish to use the
Usb interface functions the drivers must also be installed.
*/

#ifndef _AutoAnchorComms_H
#define _AutoAnchorComms_H

#if defined(__AACOMMS_EXPORTS)
#define __DLL_EXPORT __export   
#elif defined(__AACOMMS_IMPORTS) 
#define __DLL_EXPORT __import
#else
#define __DLL_EXPORT
#endif

#define MAX_SW_VERSION_STR_LEN 6
#define MAX_SW_BUILD_DATE_STR_LEN 11
#define MAX_HW_VERSION_STR_LEN 10
#define MAX_UNIT_SERIAL_NO_STR_LEN 10

//! Indicates what type of rode is currently present
typedef enum {
  rsChain, //! Chain is passing through gypsy
  rsRope   //! Rope is passing through gypsy
} TRodeStatus;

//! The movement that the gypsy is currently performing
typedef enum {
  dStopped,    //! Windlass is stopped
  dMovingDown, //! Windlass is moving down
  dMovingUp,   //! Windlass is moving up
  dLast
} TWinchDirection;

//! Error conditions that the unit may return
typedef enum {
  weNone,               //! No error detected
  weSensorWiring,       //! There is a problem with sensor wiring
  weMotorVoltage,       //! There is a problem with motor voltage wiring. Only returned on rope/chain installations
  weSolenoidUp,         //! The wire to the up solenoid is disconnected
  weSolenoidDown,       //! The wire to the down solenoid is disconnected
  weUnderVoltage,       //! The voltage applied to the unit is too low
  weSmallSignal,        //! Signal picked up by sensor is to small. Magnet or sensor may be mounted incorrectly
  weExcessiveSignal,    //! Signal picked up by sensor is to large. Magnet or sensor may be mounted incorrectly
  weSensorPulseTimeout, //! The winch is moving, but no sensor pulses are detected. There may be a problem with sensor wiring, sensor mounting or magnet mounting
  weNotInstalled,       //! All wires to the unit are disconnected
  weUnknown             //! An unknown error occured
} TWinchError;

//! Responses to commands
typedef enum {
  cmdSuccess,           //! The command has completed successfully
  cmdFail,              //! The command failed
  cmdInvalidParameters, //! The parameters passed to the command are invalid
  cmdWaitingResponse,   //! The command is in progress. Waiting for a response from unit. Keep polling until command complete
  cmdUserCancel         //! User cancelled the command
} TCommandResponse;

//! Possible connection states
typedef enum {
  csOK,           //! Connection is open, and AASTAT messages are being received
  csCommsTimeout, //! Connection is open, but no AASTAT messages are being received
  csIOError,      //! There was a problem with the communications channel (USB or Serial port)
  csInvalidHandle //! The handle specified is not valid
} TConnectionStatus;

//! Callback function type used for indicating a change of status
typedef void(*cbFunc)(void*);

//! Stores all of the status information for a particular winch
typedef struct {
  //! The rode deployed in meters
  double RodeDeployed;
  //! Inidicates if rode or chain is passing through the gypsy
  TRodeStatus RodeStatus;
  //! Indicates the direction the winch is moving in
  TWinchDirection Direction;
  //! Motor voltage. In volts
  double MotorVoltage;
  //! Motor speed in RPM
  double MotorSpeed;
  //! Motor load in kg
  double MotorLoad;
  //! Indicates the most recent error received from the Auto anchor. The error will
  //! stay set even after the error condition has gone away.
  TWinchError LastError;
  //! Version of the firmware in the AA601. Check VersionsValid flag before using
  char SWVersionStr[MAX_SW_VERSION_STR_LEN];
  //! Build date of the firmware in the AA601. Check VersionsValid flag before using
  char SWBuildDateStr[MAX_SW_BUILD_DATE_STR_LEN];
  //! Hardware version of the AA601. Check VersionsValid flag before using
  char HWVersionStr[MAX_HW_VERSION_STR_LEN];
  //! SWVersionStr, SWBuildDateStr & HWVersionStr are only valid if this flag is set
  BOOL VersionsValid;
  //! The system tick time when the last settings message was received
  int LastSettingsTime;
  //! The anchor profile that is active
  int AnchorProfile;
  //! The number of available anchor profiles
  int AnchorProfileCount;
  //! The circumference of the gypsy (for chain only winches)
  int ChainGypsyCircumference;
  //! The number of pulses since the last AASTAT message. Use this to check if the winch has moved
  int PulseCounts;
  //! This is the maximum rode installed on the unit
  int MaxRode;
  //! This is the number of seconds that the windlass motor has been running since the AA601 was installed. This counter cannot be cleared
  int MotorRunningTime;
  //! TRUE if unit is in demo mode (generating pulses internally), FALSE if unit is in normal mode (monitoring sensor and motor wires)
  BOOL DemoMode;
  //! This is set to TRUE if the USB connection fails (usually due to disconnection)
  BOOL UsbPortDisconnected;
} TAnchorStatus;

/**
Initialisation function. This function must be called before any of the other
functions in the library

\param StatusChangeFunc This function is called when a new AASTAT message is received.
                        Be aware that the function is not called from the main application
                        thread

\param CustData This data is passed to the StatusChangeFunc. Can be used to
                identify which winch status changed in a multi-winch environment.

\param PortName Name of the comport to open (e.g. "COM1", "COM2", ...) or serial
                number of USB interface to open (e.g. "DPNY3A6P"). If "DEMO" is
                specified the library will emulate a connection

\param DisableCommsAckMsg If this is 0, an acknowledge is sent to the aa601
                          after every AASTAT, and clearing rode by shorting
                          comms lines is disabled.
                          Any other value stops acks, and enables clearing rode
                          by shorting comms lines

\returns Handle to auto-anchor interface, or -1 when an error has occurred
*/
int __DLL_EXPORT
AAComms_Init( cbFunc StatusChangeFunc, void *CustData, char *PortName, int DisableCommsAckMsg );

/**
Stop an auto-anchor interface that was previously opened with the AAComms_Init function

\param handle The handle of the auto-anchor interface to stop

\returns True when communications stopped with no errors, False otherwise
*/
BOOL __DLL_EXPORT
AAComms_Stop( int handle );

/**
Gets the current status of the auto anchor connection

\param handle The handle of the auto-anchor connection to get status of

\retval csOK            If sentences are being received from auto-anchor
\retval csCommsTimeout  Communications has timed out
\retval csIOError       There has been an io error (e.g. USB was unplugged)
\retval csInvalidHandle For an invalid handle input
*/
TConnectionStatus __DLL_EXPORT
AAComms_GetConnectionStatus( int handle );

/**
Gets the current status of the auto anchor

\param handle The handle of the auto-anchor connection to get status of

\param status Pointer to structure to store the auto anchor status in

\retval TRUE  Status was read successfully
\retval FALSE There is an error
*/
BOOL __DLL_EXPORT
AAComms_GetStatus( int handle, TAnchorStatus *status );

/**
Clears the rode to zero
To prevent the user interface from locking up each time a command is sent, the
function will always return immediately even if the command is not complete.
You need to keep polling the function to discover when the command finishes (or fails).

\param handle The handle of the auto-anchor connection to set to zero

\retval cmdSuccess           This indicates that a previously initiated command has completed
\retval cmdFail              This indicates that a previously initiated command has failed
\retval cmdInvalidParameters The parameters of the command are invalid
\retval cmdWaitingResponse   The command is in progress. Keep polling till a success or fail is returned
*/
TCommandResponse __DLL_EXPORT
AAComms_ClearRode( int handle );

TCommandResponse __DLL_EXPORT
AAComms_DebugEnable( int handle, BOOL enable );

/**
Use this function to set NMEA message rates.
To prevent the user interface from locking up each time a command is sent, the
function will always return immediately even if the command is not complete. You
need to keep polling the function to discover when the command finishes (or fails).

\param handle The handle of the auto-anchor connection

\param MsgName Message name to set the output rate for

\param interval The desired output rate of the message in milliseconds.
                Pass 0 to turn message output off
                Pass -1 to output message immediately. Normal message rate not affected

\retval cmdSuccess           This indicates that a previously initiated command has completed
\retval cmdFail              This indicates that a previously initiated command has failed
\retval cmdInvalidParameters The parameters of the command are invalid
\retval cmdWaitingResponse   The command is in progress. Keep polling till a success or fail is returned
*/
TCommandResponse __DLL_EXPORT
AAComms_SetMsgRate( int handle, char *MsgName, int interval );

/**
Sets the winch settings
To prevent the user interface from locking up each time a command is sent, the
function will always return immediately even if the command is not complete. You
need to keep polling the function to discover when the command finishes (or fails).

\param handle The handle of the auto-anchor connection

\param AnchorProfile Anchor winch profile. Set to 0 for chain only winches

\param ChainGypsyCircumference Gypsy circumference for chain in mm * 10.
                               Ignored for profiles > 0

\param MaxRode The maximum amount of rode on board. This is not used by the AA601 at
               present (but may be a convenient place to store this value of the display
               unit)

\retval cmdSuccess           This indicates that a previously initiated command has completed
\retval cmdFail              This indicates that a previously initiated command has failed
\retval cmdInvalidParameters The parameters of the command are invalid
\retval cmdWaitingResponse   The command is in progress. Keep polling till a success or fail is returned
*/
TCommandResponse __DLL_EXPORT
AAComms_SetWinchSettings( int handle, int AnchorProfile, int ChainGypsyCircumference, int MaxRode );

/**
Reads the winch settings
To prevent the user interface from locking up each time a command is sent, the
function will always return immediately even if the command is not complete. You
need to keep polling the function to discover when the command finishes (or fails).

\param handle The handle of the auto-anchor connection

\param AnchorProfile Location to store anchor winch profile. 0 indicates a chain-only setup

\param ChainGypsyCircumference Location to store gypsy circumference for chain in mm * 10.
                               Ignored for profiles > 0

\param MaxAnchorProfile Location to store the number of available profiles on the unit

\param MaxRode Location to store the maximum amount of rode on board. This is not used by
               the AA601 at present (but may be a convenient place to store this value of the
               display unit)

\retval cmdSuccess           This indicates that a previously initiated command has completed
\retval cmdFail              This indicates that a previously initiated command has failed
\retval cmdInvalidParameters The parameters of the command are invalid
\retval cmdWaitingResponse   The command is in progress. Keep polling till a success or fail is returned
*/
TCommandResponse __DLL_EXPORT
AAComms_GetWinchSettings( int handle, int *AnchorProfile, int *ChainGypsyCircumference, int *MaxAnchorProfile, int *MaxRode );

/**
Use this function to enable or disable demo mode
To prevent the user interface from locking up each time a command is sent, the
function will always return immediately even if the command is not complete. You
need to keep polling the function to discover when the command finishes (or fails).

\param handle     The handle of the auto-anchor connection
\param demo_mode  Pass 0 to turn off demo mode, 1 to turn demo mode on

\retval cmdSuccess           This indicates that a previously initiated command has completed
\retval cmdFail              This indicates that a previously initiated command has failed
\retval cmdInvalidParameters The parameters of the command are invalid
\retval cmdWaitingResponse   The command is in progress. Keep polling till a success or fail is returned
*/
TCommandResponse __DLL_EXPORT
AAComms_SetDemoMode( int handle, int demo_mode );

/**
Reads the software and firmware versions from the device
To prevent the user interface from locking up each time a command is sent, the
function will always return immediately even if the command is not complete. You
need to keep polling the function to discover when the command finishes (or fails).

\param handle            The handle of the auto-anchor connection
\param FirmwareVersion   Location to store firmware version sring. Must be at least 20 bytes
\param FirmwareBuildDate Location to store firmware build date sring. Must be at least 20 bytes
\param HWVersion         Location to store hardware version sring. Must be at least 20 bytes

\retval cmdSuccess           This indicates that a previously initiated command has completed
\retval cmdFail              This indicates that a previously initiated command has failed
\retval cmdInvalidParameters The parameters of the command are invalid
\retval cmdWaitingResponse   The command is in progress. Keep polling till a success or fail is returned
*/
TCommandResponse __DLL_EXPORT
AAComms_GetFirmwareVersion( int handle, char *FirmwareVersion, char *FirmwareBuildDate, char *HWVersion );

#endif