/**
 * \file UsbDirect.h
 * Routines for sending and receiving with a usb - rs485 adapter accessed directly
 * through driver. These are basically convenience routines for calling FTD2XX.dll
 *
 * The device numbers of usb devices are dependant on startup / installation order.
 * The only reliable way to open a particular usb dongle is to look at all devices
 * to determine the one with the correct serial number (which is unique)
 *
 * Normally you would use UsbDirect_DeviceCount and then UsbDirect_GetDeviceSerialNumber
 * to give a user a list of USB devices. The serial numbers can then be passed to AAComms_Init
 *
 * NOTE: Any open connections cannot be discovered using this library, so close all connections
 *       before trying to enumerate the available Usb connections
 *
 * Copyright (C) 2006 RMSD Ltd
 */
#ifndef __USB_DIRECT_H
#define __USB_DIRECT_H

//! Indicates the status of the USB connection
typedef enum {
  udOK,              //! No error
  udTIMEOUT,         //! Function timed out
  udFAILED,          //! Function failed
  udIOERROR,         //! There was an IO error. Usually this occurs when the USB->RS485
                     //! adapter is unplugged by a user
  udDEVICE_NOT_FOUND //! The specified USB device was not found
} UD_STATUS;

// If you want to build these routines into a dll, then make sure __USBDIRECT_EXPORTS is defined
#if defined(__USBDIRECT_EXPORTS)
#define __DLL_EXPORT __export     
#elif defined(__USBDIRECT_IMPORTS)
#define __DLL_EXPORT __import    
#else
#define __DLL_EXPORT
#endif

/**
This routine writes the provided data to a serial port

\param handle    Handle to the serial port
\param lpBuf     Pointer to data to transmit
\param dwToWrite Size of data to send
*/
UD_STATUS __DLL_EXPORT UsbDirect_WriteBuffer( HANDLE handle, char *lpBuf, DWORD dwToWrite );

/**
This routine checks to see if a character has been received on the serial port.

\param handle    Handle to the serial port
\param lpBuf     Pointer to memory to store received character (left unchanged if no data avaiable)
\param WaitMsec  The maximum amount of time (in milli seconds) to wait for a character
*/
UD_STATUS __DLL_EXPORT UsbDirect_WaitForChar( HANDLE handle, char *lpBuf, DWORD WaitMsec );

/**
This routine opens a usb - rs485 interface

\param devId  This is the device number of the usb port. Use UsbDirect_SerialNumToDevNum to
              discover this number
\param baud   Baud rate to open port at

\returns INVALID_HANDLE_VALUE on error, otherwise the handle of the usb - 485 interface port
*/
HANDLE __DLL_EXPORT UsbDirect_Open( DWORD devId, int baud );

/**
Closes a previosly opened port

\param handle The value returned by a call to  UsbDirect_Open
*/
void __DLL_EXPORT UsbDirect_Close( HANDLE handle );

/**
Finds the number of connected usb devices
*/
DWORD __DLL_EXPORT UsbDirect_DeviceCount( void );

/**
Description: Finds the serial number of a particular usb device

\param Index The index of the usb device. Must be less than UsbDirect_DeviceCount
\param buf   Storage for serial number. Must be at least 10 characters long
*/
UD_STATUS __DLL_EXPORT UsbDirect_GetDeviceSerialNumber( DWORD Index, char *buf );

/**
Gets the description of a particular usb device

\param Index  The index of the usb device. Must be less than UsbDirect_DeviceCount
\param buf    Storage for serial number. Must be at least 10 characters long
*/
UD_STATUS __DLL_EXPORT UsbDirect_GetDeviceDescription( DWORD Index, char *buf );

/**
Finds a usb device that has the specified serial number

\param serialNum The serial number of the wanted usb device
\param devNum    Place to store the device number of the usb device (if it is found)
*/
UD_STATUS __DLL_EXPORT UsbDirect_SerialNumToDevNum( char *serialNum, DWORD *devNum );

#endif

